"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var TargetMetadata_1 = require("./TargetMetadata");
/**
 * This metadata interface contains all information about specific table.
 */
var TableMetadata = (function (_super) {
    __extends(TableMetadata, _super);
    function TableMetadata(target, nameOrIsAbstract, maybeIsAbstract) {
        _super.call(this, target);
        // ---------------------------------------------------------------------
        // Readonly Properties
        // ---------------------------------------------------------------------
        /**
         * Indicates if this table is abstract or not. Regular tables can inherit columns from abstract tables.
         */
        this.isAbstract = false;
        if (typeof nameOrIsAbstract === "string")
            this._name = nameOrIsAbstract;
        if (typeof nameOrIsAbstract === "boolean")
            this.isAbstract = nameOrIsAbstract;
        if (typeof maybeIsAbstract === "boolean")
            this.isAbstract = maybeIsAbstract;
    }
    Object.defineProperty(TableMetadata.prototype, "name", {
        // ---------------------------------------------------------------------
        // Getters
        // ---------------------------------------------------------------------
        /**
         * Table name in the database.
         */
        get: function () {
            if (this._name)
                return this._name;
            return this.entityMetadata.namingStrategy.tableName(this.target.name);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Checks if this table is inherited from another table.
     */
    TableMetadata.prototype.isInherited = function (anotherTable) {
        return Object.getPrototypeOf(this.target.prototype).constructor === anotherTable.target;
        // we cannot use instanceOf in this method, because we need order of inherited tables, to ensure that
        // properties get inherited in a right order. To achieve it we can only check a first parent of the class
        // return this.target.prototype instanceof anotherTable.target;
    };
    return TableMetadata;
}(TargetMetadata_1.TargetMetadata));
exports.TableMetadata = TableMetadata;
//# sourceMappingURL=TableMetadata.js.map